#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Author:  Dominik Gresch <greschd@gmx.ch>
# Date:    21.09.2016 11:37:00 CEST
# File:    __init__.py

"""
This module contains the encodings that are defined within iohelper.
"""

from . import default
